﻿using System;
using System.Collections.Generic;
using System.Text;

namespace InsideLinq
{
    public class QueryableString// : IEnumerable<char>, IEnumerator<char>, ISupportsQuerying
    {
        private string _value;
        public QueryableString(string value)
        {
            this._value = value;
        }

        public string Value
        {
            get { return _value; }
            set { _value = value; }
        }

        #region IEnumerable
        public QueryableString GetEnumerator()
        {
            this.Reset();
            return this;
        }

        private int _CurrentIndex;

        public char Current
        {
            get { return _value.ToCharArray(_CurrentIndex - 1, 1)[0]; }
        }

        public bool MoveNext()
        {
            if (_CurrentIndex < _value.Length)
            {
                _CurrentIndex += 1;
                return true;
            }
            return false;
        }

        public void Reset()
        {
            _CurrentIndex = 0;
            foundItems = new Dictionary<char, object>();
        }
        #endregion

        #region ISupports Querying
        public IEnumerable<char> Where(Func<char, bool> predicate)
        {
            foreach (char curString in this)
            {
                if (predicate(curString))
                    yield return curString;
            }
        }

        public IEnumerable<char> Select(Func<char, char> selector)
        {
            yield return selector(Current);
        }

        Dictionary<char, object> foundItems = new Dictionary<char, object>();
        public IEnumerable<char> Distinct()
        {
            if (!foundItems.ContainsKey(Current))
            {
                foundItems.Add(Current, null);
                yield return Current;
            }
        }

        #endregion

        #region Static Query Methods
        public static IEnumerable<char> Select(IEnumerable<char> source, Func<char, char> selector)
        {
            foreach (char curString in source)
                yield return selector(curString);
        }

        public static IEnumerable<char> Select(QueryableString source, Func<char, char> selector)
        {
            foreach (char curCar in source)
                yield return selector(curCar);
        }

        public static IEnumerable<char> Where(QueryableString source, Func<char, bool> predicate)
        {
            foreach (char curChar in source)
                if (predicate(curChar))
                    yield return curChar;
        }

        public static IEnumerable<char> Distinct<T>(IEnumerable<char> source)
        {
            Dictionary<char, object> foundItems = new Dictionary<char, object>();
            foreach (char element in source)
            {
                if (!foundItems.ContainsKey(element))
                {
                    foundItems.Add(element, null);
                    yield return element;
                }
            }
        }

        #endregion

        #region QueryExpressions
        public IEnumerable<char> WhereExpression(System.Linq.Expressions.Expression<Func<char, bool>> predicateExpression)
        {
            Console.WriteLine(predicateExpression.NodeType);
            Console.WriteLine(predicateExpression.Body.ToString());
            System.Linq.Expressions.BinaryExpression EqualsExpression = (System.Linq.Expressions.BinaryExpression)predicateExpression.Body;
            Console.WriteLine("Left: {0}, EQUALS  Right: {1}", EqualsExpression.Left, EqualsExpression.Right);

            foreach (char curString in this)
            {
                if (predicateExpression.Compile().Invoke(curString))
                    yield return curString;
            }
        }

        #endregion

        public static void PrintValues(IEnumerable<char> source, string formatter)
        {
            foreach (char curChar in source)
                Console.WriteLine(formatter, curChar);
            Console.WriteLine();
        }

    }

    interface ISupportsQuerying : IEnumerable<char>
    {
        IEnumerable<char> Select(Func<char, char> selector);
        IEnumerable<char> Where(Func<char, bool> predicate);
        IEnumerable<char> Distinct();
    }

    interface IGenericQuerying<T> : IEnumerable<T>
    {
        IEnumerable<TProj> Select<TProj>(Func<T, TProj> selector);
        IEnumerable<T> Where(Func<T, bool> predicate);
        IEnumerable<T> Distinct();
    }
}
