﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace InsideLinq
{
    static class QueryableStringExtensions
    {
        public static void PrintValues(this IEnumerable<char> source, string formatter)
        {
            foreach (char curChar in source)
                Console.WriteLine(formatter, curChar);
            Console.WriteLine();
        }

        #region Static Query Methods
        public static IEnumerable<char> Select(this IEnumerable<char> source, Func<char, char> selector)
        {
            foreach (char curString in source)
                yield return selector(curString);
        }

        public static IEnumerable<char> Select(this QueryableString source, Func<char, char> selector)
        {
            foreach (char curCar in source)
                yield return selector(curCar);
        }

        //public static IEnumerable<char> Where(this QueryableString source, Func<char, bool> predicate)
        //{
        //    foreach (char curChar in source)
        //        if (predicate(curChar))
        //            yield return curChar;
        //}

        public static IEnumerable<char> Where(this QueryableString source, System.Linq.Expressions.Expression<Func<char, bool>> predicateExpression)
        {
            Console.WriteLine(predicateExpression.NodeType);
            Console.WriteLine(predicateExpression.Body.ToString());
            System.Linq.Expressions.BinaryExpression EqualsExpression = (System.Linq.Expressions.BinaryExpression)predicateExpression.Body;
            Console.WriteLine("Left: {0}, EQUALS  Right: {1}", EqualsExpression.Left, EqualsExpression.Right);

            foreach (char curString in source)
            {
                if (predicateExpression.Compile().Invoke(curString))
                    yield return curString;
            }
        }

        public static IEnumerable<char> Distinct<T>(this IEnumerable<char> source)
        {
            Dictionary<char, object> foundItems = new Dictionary<char, object>();
            foreach (char element in source)
            {
                if (!foundItems.ContainsKey(element))
                {
                    foundItems.Add(element, null);
                    yield return element;
                }
            }
        }
        #endregion
    }
}
